// Copyright (c) 1999-2000 David Muse
// See the COPYING file for more information

#include <config.h>

#include <iostream.h>

#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/time.h>
#ifdef HAVE_UNISTD_H
	#include <unistd.h>
#endif
#include <string.h>
#ifdef HAVE_STRINGS_H
	#include <strings.h>
#endif
#include <sys/socket.h>

#include <rudiments/clientsocket.h>

int	clientsocket::connectToServer(char *hostname, 
					unsigned short int port, 
					int retrytime, int tries) {

	// check for an uninitialized socket
	if (sock>-1) {
		if (showerrors) {
			cerr << "error: already connected" << endl;
		}
		return 0;
	}

	if (!(protocol=getprotobyname("tcp"))) {
		if (showerrors) {
			cerr << "error: getprotobyname failed: ";
			cerr << strerror(errno) << endl;
		}
		return 0;
	}

	hostentry=NULL;
	hostentry=gethostbyname(hostname);
	if (!hostentry) {
		if (showerrors) {
			cerr << "error: gethostbyname failed: ";
			cerr << strerror(errno) << endl;
		}
		return 0;
	}

	bzero((void *)&serversin,sizeof(serversin));
	serversin.sin_family=hostentry->h_addrtype;
	serversin.sin_port=htons(port);


	// try to connect to each of the addresses that came back
	int	counter=0;
	int	failurepoint=0;
	while (counter<tries || !tries) {

		int	addressindex=0;
		while (hostentry->h_addr_list[addressindex]) {

			bcopy((void *)hostentry->h_addr_list[addressindex],
				(void *)&serversin.sin_addr,
				hostentry->h_length);
			addressindex++;
	
			if ((sock=socket(AF_INET,SOCK_STREAM,
						protocol->p_proto))>-1) {

				if (connect(sock,(struct sockaddr *)&serversin,
						sizeof(serversin))!=-1) {
					return 1;
				} else {
					failurepoint=0;
					closeSocket();
				}
			} else {
				failurepoint=1;
			}
		}
		counter++;
		sleep(retrytime);
	}

	// if we're here, the connect failed, display the correct message
	if (showerrors) {
		if (failurepoint) {
			cerr << "error: inet connect failed: ";
			cerr << strerror(errno) << endl;
			cerr << "	trying: ";
			cerr << hostname << ":";
			cerr << port << endl;
		} else {
			cerr << "error: inet socket creation failed: ";
			cerr << strerror(errno) << endl;
			cerr << "	trying: ";
			cerr << hostname << ":";
			cerr << port << endl;
		}
	}
	return 0;
}

int	clientsocket::connectToServer(char *port, int retrytime, int tries) {

	// check for an uninitialized socket
	if (sock>-1) {
		if (showerrors) {
			cerr << "error: already connected" << endl;
		}
		return 0;
	}

	serversun.sun_family=AF_UNIX;
	strcpy(serversun.sun_path,port);

	int	counter=0;
	int	failurepoint=0;
	while (counter<tries || !tries) {

		if ((sock=socket(AF_UNIX,SOCK_STREAM,0))>-1) {

			if (connect(sock,
					(struct sockaddr *)&serversun,
					sizeof(serversun))!=-1) {
				return 1;
			} else {
				failurepoint=0;
				closeSocket();
			}
		} else {
			failurepoint=1;
		}
		counter++;
		sleep(retrytime);
	}

	// if we're here, the connect failed, display the correct message
	if (showerrors) {
		if (failurepoint) {
			cerr << "error: unix connect failed: ";
			cerr << strerror(errno) << endl;
			cerr << "	trying: " << port << endl;
		} else {
			cerr << "error: unix socket creation failed: ";
			cerr << strerror(errno) << endl;
			cerr << "	trying: " << port << endl;
		}
	}
	return 0;
}

int	clientsocket::closeConnection() {
	return closeSocket();
}
