// Copyright (c) 1999-2000 David Muse
// See the COPYING file for more information

#include <config.h>

#include <iostream.h>
#include <fstream.h>
#include <strstream.h>

#include <sys/types.h>
#ifdef HAVE_UNISTD_H
	#include <unistd.h>
#endif
#include <stdlib.h>
#include <sys/stat.h>
#include <pwd.h>
#include <grp.h>
#ifdef HAVE_SYS_WAIT_H
	#include <sys/wait.h>
#endif

#include <rudiments/daemonprocess.h>

#ifdef __GNUC__
static	signalhandler	*daemonprocess::termhandler;
static	signalhandler	*daemonprocess::inthandler;
static	void		(*daemonprocess::shutdownfunc)(int);
static	signalhandler	*daemonprocess::deadchildhandler;
#endif

daemonprocess::daemonprocess() {

	deadchildhandler=new signalhandler();
	termhandler=new signalhandler();
	inthandler=new signalhandler();

	deadchildhandler->setHandler((void *)waitOnChildren);
	deadchildhandler->addFlag(SA_NOCLDSTOP);
	deadchildhandler->setSignal(SIGCHLD);
	deadchildhandler->handleSignal();
}

daemonprocess::~daemonprocess() {
	waitOnChildren();

	delete deadchildhandler;
	delete termhandler;
	delete inthandler;
}

int	daemonprocess::checkForPidFile(char *filename) {
	ifstream	pidfile(filename);
	strstream	pidstring;
	pidstring << pidfile.rdbuf() << ends;
	if (pidstring.str()[0]) {
		return atoi(pidstring.str());
	}
	return 0;
}

void	daemonprocess::createPidFile(char *filename) {
	ofstream	pidfile(filename);
	pidfile << getpid() << ends;
}

void	daemonprocess::detach() {

	// fork and let the parent process exit
	if (fork()) {
		_exit(0);
	}
	
	// become process group and session group leader 
	// with no controlling terminal
	setsid();

	// change directory to root to avoid keeping any directories in use
	chdir("/");

	// Set umask such that files are created 666 and directories 777.
	// This way we can change them to whatever we like using chmod().
	// We want to avoid inheriting a umask which wouldn't give us write
	// permissions to files we create.
	umask(0);
}

void	daemonprocess::handleShutDown(void *shutdownfunction) {

	shutdownfunc=(void(*)(int))shutdownfunction;

	termhandler->setHandler((void *)shutDown);
	termhandler->setSignal(SIGTERM);
	termhandler->handleSignal();

	inthandler->setHandler((void *)shutDown);
	inthandler->setSignal(SIGINT);
	inthandler->handleSignal();
}

void	daemonprocess::waitOnChildren() {
	while(waitpid(-1,NULL,WNOHANG)>0);
}


void	daemonprocess::shutDown() {
	(*shutdownfunc)(0);
}

int	daemonprocess::runAsUser(char *username) {
	struct	passwd	*ent=getpwnam(username);
	if (!ent) {
		return 0;
	}
	return runAsUserId(ent->pw_uid);
}

int	daemonprocess::runAsUserId(uid_t uid) {
	return setreuid(uid,uid);
}

int	daemonprocess::runAsGroup(char *groupname) {
	struct	group	*ent=getgrnam(groupname);
	if (!ent) {
		return 0;
	}
	return runAsGroupId(ent->gr_gid);
}

int	daemonprocess::runAsGroupId(gid_t gid) {
	return setregid(gid,gid);
}
