// Copyright (c) 2000 David Muse
// See the COPYING file for more information

#include <config.h>

#include <rudiments/datetime.h>
#include <stdlib.h>
#include <string.h>
#ifdef HAVE_STRINGS
	#include <strings.h>
#endif
#include <stdio.h>

datetime::datetime() {
	epoch=time(NULL);
	memcpy((void *)&timestruct,(void *)localtime(&epoch),
						sizeof(struct tm));
	timestring=NULL;
}

datetime::datetime(char *timestring) {

	this->timestring=new char[strlen(timestring)+1];
	strcpy(this->timestring,timestring);

	// get the date
	timestruct.tm_mon=atoi(timestring)-1;
	if (timestruct.tm_mon<0) {
		timestruct.tm_mon=0;
	}
	timestruct.tm_mday=atoi(timestring+3);
	if (timestruct.tm_mday<1) {
		timestruct.tm_mday=1;
	}
	timestruct.tm_year=atoi(timestring+6)-1900;

	// get the time
	timestruct.tm_hour=atoi(timestring+11);
	if (timestruct.tm_hour<0) {
		timestruct.tm_hour=0;
	}
	timestruct.tm_min=atoi(timestring+14);
	if (timestruct.tm_min<1) {
		timestruct.tm_min=1;
	}
	timestruct.tm_sec=atoi(timestring+17);
	timestruct.tm_wday=0;
	timestruct.tm_yday=0;

	epoch=mktime(&timestruct);
}

datetime::datetime(tm *timestruct) {
	memcpy((void *)&(this->timestruct),(void *)timestruct,
						sizeof(struct tm));
	epoch=mktime(&this->timestruct);
	timestring=NULL;
}

datetime::datetime(time_t *epoch) {
	memcpy((void *)&timestruct,(void *)localtime(epoch),
						sizeof(struct tm));
	this->epoch=*epoch;
	timestring=NULL;
}

datetime::~datetime() {
	if (timestring) {
		delete timestring;
	}
}

int	datetime::getHour() {
	return timestruct.tm_hour;
}

int	datetime::getMinutes() {
	return timestruct.tm_min;
}

int	datetime::getSeconds() {
	return timestruct.tm_sec;
}

int	datetime::getMonth() {
	return timestruct.tm_mon+1;
}

int	datetime::getDayOfMonth() {
	return timestruct.tm_mday;
}

int	datetime::getDayOfWeek() {
	return timestruct.tm_wday;
}

int	datetime::getDayOfYear() {
	return timestruct.tm_yday+1;
}

int	datetime::getYear() {
	return timestruct.tm_year+1900;
}

int	datetime::isDaylightSavingsTime() {
	return timestruct.tm_isdst;
}

char	*datetime::getTimeZone() {
	#ifdef HAS_TM_ZONE
		return (char *)timestruct.tm_zone;
	#elif HAS___TM_ZONE
		return (char *)timestruct.__tm_zone;
	#elif HAS_TM_NAME
		return timestruct.tm_name;
	#else
		return "";
	#endif
}

long	datetime::getTimeZoneOffset() {
	#ifdef HAS_TM_GMTOFF
		return timestruct.tm_gmtoff;
	#elif HAS___TM_GMTOFF
		return timestruct.__tm_gmtoff;
	#elif HAS_TM_TZADJ
		return timestruct.tm_tzadj;
	#else
		return 0;
	#endif
}

time_t	datetime::getEpoch() {
	return epoch;
}

char	*datetime::getString() {
	if (!timestring) {
		timestring=new char[20];
		sprintf(timestring,"%02d/%02d/%d %02d:%02d:%02d",
				timestruct.tm_mon+1,
				timestruct.tm_mday,
				timestruct.tm_year+1900,
				timestruct.tm_hour,
				timestruct.tm_min,
				timestruct.tm_sec);
	}
	return timestring;
}

tm	*datetime::getTm() {
	return &timestruct;
}
