// Copyright (c) 1999-2000 David Muse
// See the COPYING file for more information

#include <config.h>

#include <iostream.h>

#include <errno.h>
#include <stdlib.h>

#include <sys/types.h>
#include <netinet/in.h>

#include <sys/stat.h>
#include <sys/time.h>
#ifdef HAVE_UNISTD_H
	#include <unistd.h>
#endif
#include <string.h>
#ifdef HAVE_STRINGS_H
	#include <strings.h>
#endif

#define _XPG4_2
#include <sys/socket.h>
#undef _XPG4_2

#include <sys/uio.h>
#include <limits.h>

#include <rudiments/genericsocket.h>

genericsocket::genericsocket(int descriptor) {
	unixsocket=0;
	inetsocket=0;

	sock=descriptor;

	retryinterruptedreads=0;
	retryinterruptedwrites=0;

	showerrors=0;
}

genericsocket::genericsocket() {
	unixsocket=0;
	inetsocket=0;

	sock=-1;

	retryinterruptedreads=0;
	retryinterruptedwrites=0;

	showerrors=0;
}

genericsocket::~genericsocket() {
	if (sock>-1) {
		closeSocket();
	}
}

void	genericsocket::showErrors() {
	showerrors=1;
}

void	genericsocket::dontShowErrors() {
	showerrors=0;
}

int	genericsocket::closeSocket() {

	// check for an uninitialized socket
	if (sock==-1) {
		if (showerrors) {
			cerr << "error: socket is uninitialized";
			cerr << strerror(errno) << endl;
		}
		return 0;
	}

	// close the socket
	if (close(sock)) {
		if (showerrors) {
			cerr << "error: close failed: ";
			cerr << strerror(errno) << endl;
		}
		return 0;
	}

	sock=-1;

	if (inetsocket) {
		inetsocket=0;
	} else {
		unixsocket=0;
	}
	return 1;
}

void	genericsocket::retryInterruptedReads() {
	retryinterruptedreads=1;
}

void	genericsocket::dontRetryInterruptedReads() {
	retryinterruptedreads=0;
}

void	genericsocket::retryInterruptedWrites() {
	retryinterruptedwrites=1;
}

void	genericsocket::dontRetryInterruptedWrites() {
	retryinterruptedwrites=0;
}

int	genericsocket::writeData(unsigned short number) {
	unsigned short	temp=htons(number);
	return (int)write(sock,(void *)&temp,sizeof(unsigned short));
}

int	genericsocket::writeData(unsigned long number) {
	unsigned long	temp=htonl(number);
	return (int)write(sock,(void *)&temp,sizeof(unsigned long));
}

int	genericsocket::writeData(float number) {
	return (int)write(sock,(void *)&number,sizeof(float));
}

int	genericsocket::writeData(double number) {
	return (int)write(sock,(void *)&number,sizeof(double));
}

int	genericsocket::writeData(char character) {
	return (int)write(sock,&character,sizeof(char));
}

int	genericsocket::writeData(char *string) {
	return (int)write(sock,string,strlen(string));
}

int	genericsocket::writeData(char *string, size_t size) {
	return (int)write(sock,string,size);
}

int	genericsocket::writeData(void *buffer, size_t size) {
	return (int)write(sock,(void *)buffer,size);
}

int	genericsocket::readData(unsigned short *buffer) {
	unsigned short	temp;
	int	retval=(int)safeRead(sock,(void *)&temp,
					sizeof(unsigned short));
	*buffer=ntohs(temp);
	return retval;
}

int	genericsocket::readData(unsigned long *buffer) {
	unsigned long	temp;
	int	retval=(int)safeRead(sock,(void *)&temp,
					sizeof(unsigned long));
	*buffer=ntohl(temp);
	return retval;
}

int	genericsocket::readData(float *buffer) {
	return (int)safeRead(sock,(void *)buffer,sizeof(float));
}

int	genericsocket::readData(double *buffer) {
	return (int)safeRead(sock,(void *)buffer,sizeof(double));
}

int	genericsocket::readData(char *buffer, size_t size) {
	return (int)safeRead(sock,(void *)buffer,size);
}

int	genericsocket::readData(void *buffer, size_t size) {
	return (int)safeRead(sock,buffer,size);
}

int	genericsocket::readData(strstream *buffer, char *terminator) {
	int	filedescriptor=sock;
	#include "readfromfileptr.C"
	return totalread;
}

ssize_t	genericsocket::safeRead(int fd, void *buf, ssize_t count) {

	// handle cases where count>SSIZE_MAX by only 
	// reading SSIZE_MAX at a time
	ssize_t	realcount=count;
	if (count>SSIZE_MAX) {
		realcount=SSIZE_MAX;
	}

	ssize_t	retval;
	for (;;) {
		errno=0;
		retval=read(fd,buf,realcount);
		if (retval!=realcount) {
			if (retryinterruptedreads && errno==EINTR) {
				// if we got an EINTR, then we need to
				// retry the read
				continue;
			} else if (retval==0 && errno==0) {
				// eof condition
				return 0;
			} else if (retval>-1 && errno==0) {
				// If we got no error, just didn't get
				// all of the data, we need to get the
				// rest.
				return retval+safeRead(fd,buf+retval,
							realcount-retval);
			} else if (errno && showerrors) {
				cerr << "error: read failed: ";
				cerr << strerror(errno) << endl;
			}
		}
		break;
	}

	// handle cases where count>SSIZE_MAX by continuing to read
	if (realcount<count) {
		return retval+safeRead(fd,buf+retval,count-retval);
	}

	return retval;
}

ssize_t	genericsocket::safeWrite(int fd, void *buf, ssize_t count) {

	ssize_t	retval;
	for (;;) {
		errno=0;
		retval=write(fd,buf,count);
		if (retval!=count) {
			if (retryinterruptedwrites && errno==EINTR) {
				continue;
			} else if (errno && showerrors) {
				cerr << "error: write failed: ";
				cerr << strerror(errno) << endl;
			}
		}
		break;
	}
	return retval;
}

int	genericsocket::passFileDescriptor(int descriptor) {

	// have to use sendmsg to pass a file descriptor. 
	// sendmsg can only send a msghdr
	struct	msghdr	messageheader;

	// no need to name the message
	messageheader.msg_name=NULL;
	messageheader.msg_namelen=0;

	// must send at least 1 iovector with 1 byte of real data
	struct iovec	iovector[1];
	iovector[0].iov_base=(void *)" ";
	iovector[0].iov_len=sizeof(char);
	messageheader.msg_iov=iovector;
	messageheader.msg_iovlen=1;

	// use other parts of the msghdr structure to send the descriptor
	#ifdef HAVE_CMSGHDR

		// new-style:
		// The descriptor is passed in the cmsghdr part of a 
		// control_union.

		#ifdef HAVE_CMSG_SPACE
		// Multiple descriptors could be passed, but we're 
		// just going to send 1.
		union {
			struct cmsghdr	cm;
			char		control[CMSG_SPACE(sizeof(int))];
		} control_union;
		messageheader.msg_control=control_union.control;
		messageheader.msg_controllen=sizeof(control_union.control);

		struct cmsghdr	*cmptr;
		cmptr=CMSG_FIRSTHDR(&messageheader);
		cmptr->cmsg_level=SOL_SOCKET;
		cmptr->cmsg_type=SCM_RIGHTS;
		cmptr->cmsg_len=CMSG_LEN(sizeof(int));
		#else
		int	controllen=sizeof(struct cmsghdr)+sizeof(int);
		struct cmsghdr	*cmptr=(struct cmsghdr *)malloc(controllen);
		cmptr->cmsg_level=SOL_SOCKET;
		cmptr->cmsg_type=SCM_RIGHTS;
		cmptr->cmsg_len=controllen;
		messageheader.msg_control=(caddr_t)cmptr;
		messageheader.msg_controllen=controllen;
		#endif
		*((int *)CMSG_DATA(cmptr))=descriptor;
	#else
		// old-style:
		// the descriptor is passed in the accrights/accrightslen 
		// section of the msghdr
		messageheader.msg_accrights=(caddr_t)&descriptor;
		messageheader.msg_accrightslen=sizeof(int);
	#endif

	// finally, send the msghdr
	if (sendmsg(sock,&messageheader,0)==-1) {
		if (showerrors) {
			cerr << "error: sendmsg failed: ";
			cerr << strerror(errno) << endl;
		}
		return 0;
	}
	return 1;
}

int	genericsocket::receiveFileDescriptor(int *descriptor) {

	// have to use recvmsg to receive a file descriptor. 
	// recvmsg can only send a msghdr
	struct msghdr	messageheader;

	// no need to name the message
	messageheader.msg_name=NULL;
	messageheader.msg_namelen=0;

	// the process that's going to handoff it's socket will also 
	// send a single iovector with a single byte of data in it, 
	// so we'll receive that too
	struct iovec	iovector[1];
	char		ptr;
	iovector[0].iov_base=&ptr;
	iovector[0].iov_len=sizeof(char);
	messageheader.msg_iov=iovector;
	messageheader.msg_iovlen=1;

	#ifdef HAVE_CMSGHDR
		#ifdef HAVE_CMSG_SPACE
		// new-style:
		// The descriptor is received in the cmsghdr part 
		// of a control_union.
		union {
			struct cmsghdr	cm;
			char		control[CMSG_SPACE(sizeof(int))];
		} control_union;
		messageheader.msg_control=control_union.control;
		messageheader.msg_controllen=sizeof(control_union.control);
		#else
		int	controllen=sizeof(struct cmsghdr)+sizeof(int);
		struct cmsghdr	*cmptr=(struct cmsghdr *)malloc(controllen);
		messageheader.msg_control=(caddr_t)cmptr;
		messageheader.msg_controllen=controllen;
		#endif
	#else
		// old-style
		// The descriptor is passed in the accrights
		int	newdescriptor;
		messageheader.msg_accrights=(caddr_t)&newdescriptor;
		messageheader.msg_accrightslen=sizeof(int);
	#endif

	// receive the msghdr
	if (recvmsg(sock,&messageheader,0)==-1) {
		if (showerrors) {
			cerr << "error: recvmsg failed: ";
			cerr << strerror(errno) << endl;
		}
		return 0;
	}

	// if we got valid data, set the passed-in descriptor to the 
	// descriptor we received and return success
	#ifdef HAVE_CMSGHDR

		#ifdef HAVE_CMSG_SPACE
		struct cmsghdr	*cmptr=CMSG_FIRSTHDR(&messageheader);
		if (cmptr && cmptr->cmsg_len==CMSG_LEN(sizeof(int)) &&
				cmptr->cmsg_level==SOL_SOCKET &&
				cmptr->cmsg_type==SCM_RIGHTS) {

			// if we got good data, set the desctiptor and return
			*descriptor=*((int *)CMSG_DATA(cmptr));
			return 1;

		} else {

			// if we got bad data, be specific about what was
			// wrong, this will help debug problems with different
			// platforms
			if (showerrors) {
				if (!cmptr) {
					cerr << "error: null cpmtr" << endl;
				} else {
					if (cmptr->cmsg_len!=
							CMSG_LEN(sizeof(int))) {
						cerr << "error: got cmsg_len=";
				       		cerr << cmptr->cmsg_len;
				       		cerr << " instead of ";
						cerr << CMSG_LEN(sizeof(int));
						cerr << endl;
					}
					if (cmptr->cmsg_level!=SOL_SOCKET) {
						cerr << "error: ";
						cerr << "got cmsg_level=";
						cerr << cmptr->cmsg_level;
						cerr << " instead of";
						cerr << SOL_SOCKET << endl;
					}
					if (cmptr->cmsg_type!=SCM_RIGHTS) {
						cerr << "error: got cmsg_type=";
						cerr << cmptr->cmsg_type;
						cerr << " instead of ";
						cerr << SCM_RIGHTS << endl;
					}
				}
			}
		}
		#else
		if (messageheader.msg_controllen==controllen) {
			*descriptor=*((int *)CMSG_DATA(cmptr));
			return 1;
		} else {
			cerr << "error: got msg_controllen=";
			cerr << messageheader.msg_controllen;
			cerr << " instead of ";
			cerr << controllen << endl;
		}
		#endif
	#else
		if (messageheader.msg_accrightslen==sizeof(int)) {
			*descriptor=newdescriptor;
			return 1;
		}
	#endif

	// if we're here then we must have received some bad data
	if (showerrors) {
		cerr << "error: received bad data" << endl;
	}
	return 0;
}

int	genericsocket::getDescriptor() {
	return sock;
}

int	genericsocket::isInetSocket() {
	return inetsocket;
}

int	genericsocket::isUnixSocket() {
	return unixsocket;
}
