// Copyright (c) 1999-2000 David Muse
// See the COPYING file for more information

#include <config.h>

#include <iostream.h>

#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/time.h>
#ifdef HAVE_UNISTD_H
	#include <unistd.h>
#endif
#include <string.h>
#ifdef HAVE_STRINGS_H
	#include <strings.h>
#endif

#include <rudiments/serversocketpool.h>

serversocketpool::serversocketpool() {
	first=NULL;
	last=NULL;
	current=NULL;
}

serversocketpool::~serversocketpool() {
	removeAllSockets();
}

void	serversocketpool::addSocket(serversocket *sock) {
	if (last) {
		last->next=new socketnode;
		last->next->sock=sock;
		last->next->previous=last;
		last->next->next=NULL;
		last=last->next;
	} else {
		first=new socketnode;
		first->sock=sock;
		first->previous=NULL;
		first->next=NULL;
		last=first;
	}
}

int	serversocketpool::waitForClientConnection(
			int timeoutsec, int timeoutusec,
			serversocket **whichsock, genericsocket **clientsock) {

	// if there are no sockets attached to this server, return failure
	if (!first) {
		return 0;
	}

	// set up the timeout
	timeval	*tv;
	if (timeoutsec>-1 && timeoutusec>-1) {
		tv=new timeval;
		tv->tv_sec=timeoutsec;
		tv->tv_usec=timeoutusec;
	} else {
		tv=NULL;
	}


	// set up list of sockets to select() on
	fd_set	socklist;
	FD_ZERO(&socklist);
	current=first;
	int	largest=0;
	while (current) {
		int	sock=current->sock->getDescriptor();
		if (sock>largest) {
			largest=sock;
		}
		FD_SET(sock,&socklist);
		current=current->next;
	}

	// wait for one of the sockets to be ready
	int	selectresult;
	int	retval=-1;
	for (;;) {

		selectresult=select(largest+1,&socklist,NULL,NULL,tv);

		// handle timeout
		if (!selectresult) {

			retval=0;
			break;

		// handle non-error, on error, select again
		} else if (selectresult>-1) {

			// figure out which socket was ready and accept on it
			current=first;
			while (current) {
				if (FD_ISSET(current->sock->
						getDescriptor(),&socklist)) {
					*whichsock=current->sock;
					*clientsock=current->sock->
						acceptClientConnection();
					break;
				}
				current=current->next;
			}
			retval=1;
			break;
		}
	}

	// clean up
	if (tv) {
		delete tv;
	}

	return retval;
}

int	serversocketpool::removeSocket(serversocket *sock) {
	current=first;
	while (current) {
		if (current->sock==sock) {
			if (current->next) {
				current->next->previous=current->previous;
			}
			if (current->previous) {
				current->previous->next=current->next;
			}
			if (current==first) {
				first=current->next;
			}
			if (current==last) {
				last=current->previous;
			}
			delete current;
			return 1;
		}
		current=current->next;
	}
	return 0;
}

void	serversocketpool::removeAllSockets() {
	current=first;
	while (current) {
		last=current->next;
		delete current;
		current=last;
	}
}
