// Copyright (c) 1999-2000 David Muse
// See the COPYING file for more information

#include <config.h>

#ifdef HAVE_UNISTD_H
	#include <unistd.h>
#endif
#include <stddef.h>

#include <rudiments/signalclasses.h>



// signalset methods
int	signalset::addSignal(int signum) {
	return sigaddset(&sigset,signum);
}

int	signalset::addAllSignals() {
	return sigfillset(&sigset);
}

int	signalset::removeSignal(int signum) {
	return sigdelset(&sigset,signum);
}

int	signalset::removeAllSignals() {
	return sigemptyset(&sigset);
}

int	signalset::signalIsInSet(int signum) {
	return sigismember(&sigset,signum);
}

sigset_t	*signalset::getSignalSet() {
	return &sigset;
}




// signalmanager methods
int	signalmanager::sendSignal(pid_t processid, int signum) {
	return kill(processid,signum);
}

int	signalmanager::raiseSignal(int signum) {
	return raise(signum);
}

int	signalmanager::ignoreSignals(sigset_t *sigset) {
	return sigprocmask(SIG_SETMASK,sigset,NULL);
}

int	signalmanager::waitForSignals(sigset_t *sigset) {
	return sigsuspend(sigset);
}

int	signalmanager::examineBlockedSignals(sigset_t *sigset) {
	return sigpending(sigset);
}




// signalhandler methods
signalhandler::signalhandler() {
	handlerstruct=new (struct sigaction);
	removeAllSignalsFromMask();
	removeAllFlags();
}

signalhandler::signalhandler(int signum, void *handler) {
	handlerstruct=new (struct sigaction);
	removeAllSignalsFromMask();
	removeAllFlags();
	setSignal(signum);
	setHandler(handler);
}

signalhandler::~signalhandler() {
	delete handlerstruct;
}

void	signalhandler::setSignal(int signum) {
	signalnumber=signum;
}

void	signalhandler::setHandler(void *handler) {
	#ifdef SIGNAL_HANDLER_INT
		handlerstruct->sa_handler=(void(*)(int))handler;
	#else
		handlerstruct->sa_handler=(void(*)(void))handler;
	#endif
}

void	signalhandler::removeAllFlags() {
	handlerstruct->sa_flags=0;
}

void	signalhandler::addFlag(int flag) {
	handlerstruct->sa_flags|=flag;
}

int	signalhandler::addAllSignalsToMask() {
	return sigfillset(&handlerstruct->sa_mask);
}

int	signalhandler::addSignalToMask(int signum) {
	return sigaddset(&handlerstruct->sa_mask,signum);
}

int	signalhandler::removeSignalFromMask(int signum) {
	return sigdelset(&handlerstruct->sa_mask,signum);
}

int	signalhandler::removeAllSignalsFromMask() {
	return sigemptyset(&handlerstruct->sa_mask);
}

int	signalhandler::signalIsInMask(int signum) {
	return sigismember(&handlerstruct->sa_mask,signum);
}

sigset_t	*signalhandler::getMask() {
	return &handlerstruct->sa_mask;
}

int		signalhandler::getFlags() {
	return handlerstruct->sa_flags;
}

int	signalhandler::handleSignal() {
	return sigaction(signalnumber,handlerstruct,(struct sigaction *)NULL);
}
